#pragma once
#include <map>
#include <string>
#include <fstream>
#include <iostream>
#include <iterator>
#include <iomanip>
#include <algorithm>
#include <string>

struct Date {
	int day;
	int month;
	int year;

	bool operator ==(const Date& to_compare) { return (day == to_compare.day) && (month == to_compare.month) && (year == to_compare.year); };
	bool operator <(const Date& to_compare) {
		if (year < to_compare.year) {
			return true;
		}
		else if (year == to_compare.year) {
			if (month < to_compare.month) {
				return true;
			}
			else if (month == to_compare.month) {
				if (day < to_compare.day) {
					return true;
				}
				else {
					return false;
				}
			}
			else {
				return false;
			}
		}
		else {
			return false;
		}
	}
	int day_in_month(int year, int month) const {
		int days[12] = { 31,28,31,30,31,30,31,31,30,31,30,31 };
		if (month == && ((year % 4 == 0) && (year % 100 != 0) || (year % 400 == 0))) {
			return 29;
		}
		return days[month];
	}

	Date operator+(int day) const {
		Date ret(*this);
		while (ret.day + day > day_in_month(ret.year, ret.month)) {
			day = day - (day_in_month(ret.year, ret.month) - ret.day);
			ret.day = 0;
			if (ret.month == 12) {
				ret.year++;
				ret.month = 0;
			}
			ret.month++;
		}
		ret.day += day;

		return ret;
	}


	friend std::ostream& operator << (std::ostream& out, const Date& p_date) {
		out << p_date.day << "." << p_date.month << "." << p_date.year;
		return out;
	};
	friend std::istream& operator >> (std::istream& in, Date& p_date) {
		std::cout << "day = ";
		in >> p_date.day;
		std::cout << "month = ";
		in >> p_date.month;
		std::cout << "year = ";
		in >> p_date.year;
		return in;
	}
};

struct Production {
	int storage_id;
	int product_code;
	char product_name[20];
	Date arrival_date;
	int shelf_life;
	int amount_of_products;
	double unit_price;

	bool operator ==(const Production& prod_to_comp) {
		return storage_id == prod_to_comp.storage_id &&
			product_code == prod_to_comp.product_code &&
			strcmp(product_name, prod_to_comp.product_name) &&
			arrival_date == prod_to_comp.arrival_date &&
			shelf_life == prod_to_comp.shelf_life &&
			amount_of_products == prod_to_comp.amount_of_products &&
			unit_price == prod_to_comp.unit_price;
	}

	Production() = default;

	Production(const Production& elem) {
		storage_id = elem.storage_id;
		product_code = elem.product_code;
		strcpy_s(product_name, elem.product_name);
		arrival_date = elem.arrival_date;
		shelf_life = elem.shelf_life;
		amount_of_products = elem.amount_of_products;
		unit_price = elem.unit_price;
	}

	Production& operator =(const Production& elem) {
		storage_id = elem.storage_id;
		product_code = elem.product_code;
		strcpy_s(product_name, elem.product_name);
		arrival_date = elem.arrival_date;
		shelf_life = elem.shelf_life;
		amount_of_products = elem.amount_of_products;
		unit_price = elem.unit_price;
		return *this;
	}


	friend std::ostream& operator <<(std::ostream& out, const Production& P) {
		out << std::setw(40) << std::left << P.storage_id << std::setw(45) << std::left << P.product_code
			<< std::setw(50) << std::left << P.product_name << P.arrival_date
			<< std::setw(50) << std::right << P.shelf_life << std::setw(40) << std::right << P.amount_of_products
			<< std::setw(50) << std::right << P.unit_price << std::endl;

		return out;
	}

	friend std::istream& operator >>(std::istream& in, Production& P) {
		std::cout << "Storage id = ";
		in >> P.storage_id;
		std::cout << "Product code = ";
		in >> P.product_code;
		std::cout << "Product name = ";
		in >> P.product_name;
		std::cout << "Arrival date:" << std::endl;
		in >> P.arrival_date;
		std::cout << "Shelf life in days = ";
		in >> P.shelf_life;
		std::cout << "Amount of products = ";
		in >> P.amount_of_products;
		std::cout << "Unit price = ";
		in >> P.unit_price;
		return in;

	}
};

class Storage {
	std::map<size_t, Production> product;
	std::string bin_f_name;
	std::string txt_f_name;
	size_t f_amount;
	static void new_prod_insert(Production& tmp_prod);
	void load_in();

public:
	Storage(const std::string& bin_f, const std::string& txt_to_out);
	~Storage();

	void create_and_write();
	void print_data()  noexcept;
	void print_product(const std::pair<size_t, Production>& prod);
	void update_file();

	void add_product();
	void delete_product(int prod_id);
	void change_product(int prod_id);

	void search_storage_id(int id) noexcept;
	void search_product_code(int code) noexcept;
	void search_arrival_date(const Date& arr_date) noexcept;
	void search_overdue_products(const Date& curr_date) noexcept;
	void search_not_overdue_products(const Date& curr_date) noexcept;

	void load_out_txt();
};
